﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using AIOUSBNet;  // the namespace exposes the AIOUSB Class interface 

// You must also add a reference to the AIOUSBnet.dll in the project settings


namespace Sample1
{
    public partial class Form1 : Form
    {
        // One and only Device Index:
        public UInt32 DeviceIndex;
        public UInt32 numPins = 24;
        public UInt32 numPorts = 4;
        public UInt32 numCounters = 6;

        // Interval Timer fou GUI update:
        // (Note: A Windows Forms timer designed for single threaded environments not a System Timer 55 ms min res)
        static System.Windows.Forms.Timer myTimer = new System.Windows.Forms.Timer();

        // Array of controls for GUI updates:
        public CheckBox[] checkStateArray = new CheckBox[24 /*numPins*/];
        public Button[] toggleBtnArray = new Button[4 /*numPorts*/];
        public TextBox[] CountDisplay = new TextBox[6 /*numCounters*/];

        // For bitmasks:
        byte[] OutMask = new byte[] { 0, 0};
        byte[] Data = new byte[] { 0, 0, 0 };

        // State flags:
        byte bTristateAll = 0;  // 0 = false all bits not tristated
        public bool[] bModeInOut = new bool[4];
        
        public Form1()
        {
            InitializeComponent();
            int i = 0;

            // Array of the toggle mode buttons:
            toggleBtnArray[0] = btnToggle0;
            toggleBtnArray[1] = btnToggle1;
            toggleBtnArray[2] = btnToggle2;
            toggleBtnArray[3] = btnToggle3;

            CountDisplay[0] = textBox1;
            CountDisplay[1] = textBox2;
            CountDisplay[2] = textBox3;
            CountDisplay[3] = textBox4;
            CountDisplay[4] = textBox5;
            CountDisplay[5] = textBox6;


            // Array of the LED style checkboxes:
            checkStateArray[0] = checkBoxA0_1;
            checkStateArray[1] = checkBoxA0_2;
            checkStateArray[2] = checkBoxA0_3;
            checkStateArray[3] = checkBoxA0_4;
            checkStateArray[4] = checkBoxA0_5;
            checkStateArray[5] = checkBoxA0_6;
            checkStateArray[6] = checkBoxA0_7;
            checkStateArray[7] = checkBoxA0_8;
            checkStateArray[8] = checkBoxB0_9;
            checkStateArray[9] = checkBoxB0_10;
            checkStateArray[10] = checkBoxB0_11;
            checkStateArray[11] = checkBoxB0_12;
            checkStateArray[12] = checkBoxB0_13;
            checkStateArray[13] = checkBoxB0_14;
            checkStateArray[14] = checkBoxB0_15;
            checkStateArray[15] = checkBoxB0_16;
            checkStateArray[16] = checkBoxC0_17;
            checkStateArray[17] = checkBoxC0_18;
            checkStateArray[18] = checkBoxC0_19;
            checkStateArray[19] = checkBoxC0_20;
            checkStateArray[20] = checkBoxC0_21;
            checkStateArray[21] = checkBoxC0_22;
            checkStateArray[22] = checkBoxC0_23;
            checkStateArray[23] = checkBoxC0_24;


            // Initialize default values:
            DeviceIndex = AIOUSB.diFirst;   // Default to first device found 
            
            UInt32 Status;
            UInt64 SerNum;

            // Check device status:
            Status = AIOUSB.ClearDevices();  // Cleans up any orphaned indexes
            Status = AIOUSB.GetDevices();
            Status = AIOUSB.GetDeviceSerialNumber(DeviceIndex, out SerNum);

            //Byte Tristate = 0; // 0 = false all bits not tristated
	        //Byte outMaskByte = 0;  // all bit to 0 mean all ports not output hence input

            for (i = 0; i < 3; i++)
            {
                Data[i] = 0xFF;
                bModeInOut[i] = false;
            }

            Status = AIOUSB.DIO_Configure(DeviceIndex, bTristateAll, OutMask, Data);

            AIOUSB.DIO_ReadAll(DeviceIndex, Data); // we just set this validate
        }
        
        private void Form1_Load(object sender, EventArgs e)
        {
            // Called before Form is displayed Initialize resources:

            // Initialize default Device Only:
            DeviceIndex = AIOUSB.diOnly;

            // Device data:
            UInt32 Status;
            UInt32 PID = 0;
            UInt32 NameSize = 256;
            string strName = "name";
            UInt32 DIOBytes = 0;
            UInt32 Counters = 0;
            UInt64 SerNum;
            UInt16 I = 0;

            UInt32 ERROR_SUCCESS = 0;
            bool deviceIndexValid = false;

            // Get The Device Information test for valid device found:
            Status = AIOUSB.QueryDeviceInfo(DeviceIndex, out PID, ref NameSize, out strName, out DIOBytes, out Counters);
            if ((Status == ERROR_SUCCESS) && (PID >= 0x8005) && (PID <= 0x8006)) // 5 and 6 USB-DIO24 and USB-DIO24-CTR6
            {
                deviceIndexValid = true;
            }
            else
            {
                // If Only device is not valid then Launch connect device dialog box:
                // New parent aware subform:
                FormDetect DetectSubForm = new FormDetect(this);
                DetectSubForm.ShowDialog();

                Status = AIOUSB.QueryDeviceInfo(DeviceIndex, out PID, ref NameSize, out strName, out DIOBytes, out Counters);
                if ((Status == ERROR_SUCCESS) && (PID >= 0x8005) && (PID <= 0x8006)) // 5 and 6 USB-DIO24 and USB-DIO24-CTR6
                    deviceIndexValid = true;
            }

            if (!deviceIndexValid)
            {
                // No valid device found should exit
                //this.Close(); // note: commented out to allow sample to run with any card, just for debugging.
            }

            for (I = 0; I < numCounters; I++)
              AIOUSB.CTR_8254ModeLoad(DeviceIndex, 0, I, 2, 0xFFFF);

            // Add the event and the event handler for the method that will 
            // process the timer event to the timer:
            myTimer.Tick += new EventHandler(TimerEventProcessor);
            // Set the timer interval in miliseconds and start 55ms min resolution:
            myTimer.Interval = 100;
            myTimer.Start();
        }
        
        private void TimerEventProcessor(Object myObject, EventArgs myEventArgs)
        {
            // This is the method that runs when the timer event is raised:
            myTimer.Stop();

            UpdateGUIState();

            myTimer.Enabled = true;
        }
        
        private void UpdateGUIState()
        {
            UInt32 Status = 0;
            // Get data from board update display states:
            Status = AIOUSB.DIO_ReadAll(DeviceIndex, Data);

            if (Status != 0) return;


            // Test each bit and set check state:
            for (int i = 0; i < numPins; i++)
            {
                if( (Data[i/8] & ( 1 << (i % 8) )) != 0 ) // looks complicated but efficient
                    checkStateArray[i].Checked = true; // High Level
                else
                    checkStateArray[i].Checked = false; // Low Level
            }
            
            ushort pReadValue;
            for (uint j = 0; j < numCounters; j++)
            {
                AIOUSB.CTR_8254Read(DeviceIndex, 0, j, out pReadValue);
                string formatString;
                formatString = String.Format("{0,0:G}", pReadValue);
                CountDisplay[j].Text = formatString;
            }
        }
 
        private void btnTristateAll_Click(object sender, EventArgs e)
        {
            // Need to stop the timer change board and GUI:
            myTimer.Stop(); 
            
            int i = 0;
            Int32 iOutMask = 0x0000;
            Int16 isOutMask = 0x0000;

            Button btnSender = sender as Button;

            // Toggle tristate var;
            if (bTristateAll != 0) // bool byte
            {
                bTristateAll = 0;
                btnSender.Text = "Enter Tristate Mode";
                Tristatelabel.Text = "Not Tristate: I/O Buffers Enabled";
            }
            else
            {
                bTristateAll = 1;
                btnSender.Text = " Exit Tristate Mode";
                Tristatelabel.Text = "Tristate Mode: I/O Buffers Disabled";
            }

            // Recreate Outmask for Configure:
            for (i = 0; i < numPorts; i++)
                if (bModeInOut[i] == true)  // if output mode
                    iOutMask = iOutMask | (1 << i) // Int32
            ;
            isOutMask = (Int16)iOutMask;  //safe downcast?
            AIOUSB.DIO_Configure(DeviceIndex, bTristateAll, ref isOutMask, Data);

            // Restart timer thread:
            myTimer.Enabled = true;
        }

        private void btnToggle_Click(object sender, EventArgs e)
        {
            UInt32 Status = 0;
            // Button Clicks to toggle the input / output mode and display states:

            // Need to stop the timer change board and GUI:
            myTimer.Stop(); 
            
            Int32 i;
            Int32 iOutMask = 0x0000;
            Int16 isOutMask = 0x0000;

            //Get button sender Tag (index)
            UInt32 iTagindex = 0;
            Button btnSender = sender as Button;
            if (btnSender != null)
            {
                iTagindex = Convert.ToUInt32(btnSender.Tag);
            }

            // Toggle local mode value and button display states:
            if (bModeInOut[iTagindex])
            {
                bModeInOut[iTagindex] = false;
                btnSender.Text = "Toggle to Output Mode";
            }
            else
            {
                bModeInOut[iTagindex] = true;
                btnSender.Text = "Toggle to Input Mode";
            }

            // Recreate new Outmask for Configure:         
            for (i = 0; i < numPorts; i++)
                if (bModeInOut[i] == true)  // if output mode
                    iOutMask = iOutMask | (1 << i) // Int32
            ; 
            isOutMask = (Int16)iOutMask;  //safe downcast

            Status = AIOUSB.DIO_Configure(DeviceIndex, bTristateAll, ref isOutMask, Data);

            // Restart timer thread:
            myTimer.Enabled = true;
        }

        private void btnDevice_Click(object sender, EventArgs e)
        {
            // Launch connect device dialog box
            // Switch between multiple devices or reconnect

            // New parent aware subform:
            FormDetect DetectSubForm = new FormDetect(this);
            DetectSubForm.ShowDialog();
        }

        private void checkBoxPin_Click(object sender, EventArgs e)
        {
            // Need to stop the timer change board and GUI:
            myTimer.Stop();

            bool bHighLow;
            UInt32 iTagindex = 0;

            // get correct checkbox get the checked state

            //Get sender Tag (index) and if check
            CheckBox chkSender = sender as CheckBox;
            if (chkSender != null)
            {
                iTagindex = Convert.ToUInt32(chkSender.Tag);
            }
            bHighLow = chkSender.Checked;

            Int32 i;
            Int32 iOutMask = 0x0000;
            byte bByte = 1;

            i = (Int32)iTagindex;  // downcast

            iOutMask = ((Data[i / 8]) & (1 << (i % 8))); // get single bit
            if (iOutMask != 0)
                bByte = 0;

            AIOUSB.DIO_Write1(DeviceIndex, (UInt32)i, bByte);  //byte is used like a bool

            // restart timer and Gui will update:
            myTimer.Enabled = true;

        }

        private void label7_Click(object sender, EventArgs e)
        {

        }

        private void button1_Click(object sender, EventArgs e)
        {
            AIOUSB.CTR_8254ModeLoad(DeviceIndex, 0, 0, 2, 0xFFFF);
        }

        private void button2_Click(object sender, EventArgs e)
        {
            AIOUSB.CTR_8254ModeLoad(DeviceIndex, 0, 1, 2, 0xFFFF);
        }

        private void button3_Click(object sender, EventArgs e)
        {
            AIOUSB.CTR_8254ModeLoad(DeviceIndex, 1, 0, 2, 0xFFFF);
        }

        private void button4_Click(object sender, EventArgs e)
        {
            AIOUSB.CTR_8254ModeLoad(DeviceIndex, 1, 1, 2, 0xFFFF);
        }

    }
}
